@@ -6,16 +6,17 @@ import datetime
 from errors import *
 from mozdevice import devicemanagerADB, DMError
 from mozprocess import ProcessHandlerMixin
-import multiprocessing
 import os
 import re
 import platform
 import shutil
 import socket
 import subprocess
+import sys
 from telnetlib import Telnet
 import tempfile
 import time
+import traceback
 
 from emulator_battery import EmulatorBattery
 from emulator_geo import EmulatorGeo
@@ -53,6 +54,7 @@ class Emulator(object):
         self._tmp_sdcard = None
         self._tmp_userdata = None
         self._adb_started = False
+        self.remote_user_js = '/data/local/user.js'
         self.logcat_dir = logcat_dir
         self.logcat_proc = None
         self.arch = arch
@@ -250,7 +252,7 @@ class Emulator(object):
                 os.remove(self._tmp_sdcard)
                 self._tmp_sdcard = None
             return retcode
-        if self.logcat_proc:
+        if self.logcat_proc and self.logcat_proc.proc.poll() is None:
             self.logcat_proc.kill()
         return 0
 
@@ -310,7 +312,6 @@ waitFor(
         marionette.set_context(marionette.CONTEXT_CONTENT)
         marionette.delete_session()
 
-
     def connect(self):
         self._check_for_adb()
         self.start_adb()
@@ -327,6 +328,15 @@ waitFor(
         self.dm = devicemanagerADB.DeviceManagerADB(adbPath=self.adb,
                                                     deviceSerial='emulator-%d' % self.port)
 
+    def add_prefs_to_profile(self, prefs=None):
+        if not prefs:
+            prefs = ["user_pref('marionette.loadearly', true);"]
+        local_user_js = tempfile.mktemp(prefix='localuserjs')
+        self.dm.getFile(self.remote_user_js, local_user_js)
+        with open(local_user_js, 'a') as f:
+            f.write('/n'.join(prefs))
+        self.dm.pushFile(local_user_js, self.remote_user_js)
+
     def start(self):
         self._check_for_b2g()
         self.start_adb()
@@ -368,12 +378,38 @@ waitFor(
         # setup DNS fix for networking
         self._run_adb(['shell', 'setprop', 'net.dns1', '10.0.2.3'])
 
-    def _save_logcat_proc(self, filename, cmd):
-        self.logcat_proc = LogcatProc(filename, cmd)
-        self.logcat_proc.run()
-        self.logcat_proc.processOutput()
-        self.logcat_proc.waitForFinish()
-        self.logcat_proc = None
+    def setup(self, marionette, gecko_path=None, load_early=False):
+        # Wait for the system-message-listener-ready event, otherwise
+        # Bad Things happen.
+        self.wait_for_system_message(marionette)
+
+        if gecko_path:
+            if load_early:
+                # Inject prefs into the profile now, since we have to restart
+                # B2G after installing a new gecko anyway.
+                self.add_prefs_to_profile()
+            self.install_gecko(gecko_path, marionette)
+        elif load_early:
+            self.add_prefs_to_profile()
+            self.restart_b2g()
+
+        #if load_early:
+        # Temporarily enable this always until part 2 of bug 815807 lands
+        if True:
+            # If we're loading early, we have to wait for the
+            # system-message-listener-ready event again after restarting B2G.
+            # If we're not loading early, we skip this because Marionette
+            # doesn't load until after this event has fired.
+            self.wait_for_system_message(marionette)
+
+    def restart_b2g(self):
+        print 'restarting B2G'
+        self.dm.shellCheckOutput(['stop', 'b2g'])
+        time.sleep(10)
+        self.dm.shellCheckOutput(['start', 'b2g'])
+
+        if not self.wait_for_port():
+            raise TimeoutException("Timeout waiting for marionette on port '%s'" % self.marionette_port)
 
     def install_gecko(self, gecko_path, marionette):
         """
@@ -387,33 +423,52 @@ waitFor(
         push_attempts = 10
 
         print 'installing gecko binaries...'
-        # need to remount so we can write to /system/b2g
-        self._run_adb(['remount'])
-        for root, dirs, files in os.walk(gecko_path):
-            for filename in files:
-                rel_path = os.path.relpath(os.path.join(root, filename), gecko_path)
-                system_b2g_file = os.path.join('/system/b2g', rel_path)
-                for retry in range(1, push_attempts+1):
-                    print 'pushing', system_b2g_file, '(attempt %s of %s)' % (retry, push_attempts)
-                    try:
-                        self.dm.pushFile(os.path.join(root, filename), system_b2g_file)
-                        break
-                    except DMError:
-                        if retry == push_attempts:
-                            raise
 
-        print 'restarting B2G'
         # see bug 809437 for the path that lead to this madness
-        time.sleep(5)
-        self.dm.shellCheckOutput(['stop', 'b2g'])
-        time.sleep(10)
-        self.dm.shellCheckOutput(['start', 'b2g'])
-        time.sleep(5)
+        try:
+            # need to remount so we can write to /system/b2g
+            self._run_adb(['remount'])
+            for root, dirs, files in os.walk(gecko_path):
+                for filename in files:
+                    rel_path = os.path.relpath(os.path.join(root, filename), gecko_path)
+                    system_b2g_file = os.path.join('/system/b2g', rel_path)
+                    for retry in range(1, push_attempts + 1):
+                        print 'pushing', system_b2g_file, '(attempt %s of %s)' % (retry, push_attempts)
+                        try:
+                            self.dm.pushFile(os.path.join(root, filename), system_b2g_file)
+                            break
+                        except DMError:
+                            if retry == push_attempts:
+                                raise
 
-        if not self.wait_for_port():
-            raise TimeoutException("Timeout waiting for marionette on port '%s'" % self.marionette_port)
-        self.wait_for_system_message(marionette)
+            self.restart_b2g()
 
+        except (DMError, MarionetteException):
+            # Bug 812395 - raise a single exception type for these so we can
+            # explicitly catch them elsewhere.
+
+            # print exception, but hide from mozharness error detection
+            exc = traceback.format_exc()
+            exc = exc.replace('Traceback', '_traceback')
+            print exc
+
+            raise InstallGeckoError("unable to restart B2G after installing gecko")
+
+    def install_busybox(self, busybox):
+        self._run_adb(['remount'])
+
+        push_attempts = 10
+        remote_file = "/system/bin/busybox"
+        for retry in range(1, push_attempts+1):
+            print 'pushing', remote_file, '(attempt %s of %s)' % (retry, push_attempts)
+            try:
+                self.dm.pushFile(busybox, remote_file)
+                break
+            except DMError:
+                if retry == push_attempts:
+                    raise
+        self._run_adb(['shell', 'cd /system/bin; chmod 555 busybox; for x in `./busybox --list`; do ln -s ./busybox $x; done'])
+        self.dm._verifyZip()
 
     def rotate_log(self, srclog, index=1):
         """ Rotate a logfile, by recursively rotating logs further in the sequence,
@@ -435,12 +490,8 @@ waitFor(
             self.rotate_log(filename)
         cmd = [self.adb, '-s', 'emulator-%d' % self.port, 'logcat']
 
-        # We do this in a separate process because we call mozprocess's
-        # waitForFinish method to process logcat's output, and this method
-        # blocks.
-        proc = multiprocessing.Process(target=self._save_logcat_proc, args=(filename, cmd))
-        proc.daemon = True
-        proc.start()
+        self.logcat_proc = LogcatProc(filename, cmd)
+        self.logcat_proc.run()
 
     def setup_port_forwarding(self, remote_port):
         """ Set up TCP port forwarding to the specified port on the device,
