@@ -3,6 +3,8 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import socket
+import sys
+import traceback
 
 from client import MarionetteClient
 from application_cache import ApplicationCache
@@ -11,6 +13,7 @@ from errors import *
 from emulator import Emulator
 from geckoinstance import GeckoInstance
 
+
 class HTMLElement(object):
 
     CLASS = "class name"
@@ -74,7 +77,7 @@ class HTMLElement(object):
 
     def is_displayed(self):
         return self.marionette._send_message('isElementDisplayed', 'value', element=self.id)
-     
+
     @property
     def size(self):
         return self.marionette._send_message('getElementSize', 'value', element=self.id)
@@ -97,7 +100,7 @@ class Marionette(object):
                  emulator=None, sdcard=None, emulatorBinary=None,
                  emulatorImg=None, emulator_res='480x800', gecko_path=None,
                  connectToRunningEmulator=False, homedir=None, baseurl=None,
-                 noWindow=False, logcat_dir=None):
+                 noWindow=False, logcat_dir=None, busybox=None, load_early=False):
         self.host = host
         self.port = self.local_port = port
         self.bin = bin
@@ -111,7 +114,6 @@ class Marionette(object):
         self.baseurl = baseurl
         self.noWindow = noWindow
         self.logcat_dir = logcat_dir
-        self.gecko_path = gecko_path
         self._test_name = None
 
         if bin:
@@ -143,9 +145,10 @@ class Marionette(object):
         self.client = MarionetteClient(self.host, self.port)
 
         if emulator:
-            self.emulator.wait_for_system_message(self)
-        if self.gecko_path:
-            self.emulator.install_gecko(self.gecko_path, self)
+            self.emulator.setup(self, gecko_path=gecko_path,
+                                load_early=load_early)
+            if busybox:
+                self.emulator.install_busybox(busybox)
 
     def __del__(self):
         if self.emulator:
@@ -155,6 +158,26 @@ class Marionette(object):
         for qemu in self.extra_emulators:
             qemu.emulator.close()
 
+    @classmethod
+    def getMarionetteOrExit(cls, *args, **kwargs):
+        try:
+            m = cls(*args, **kwargs)
+            return m
+        except InstallGeckoError:
+            # Bug 812395 - the process of installing gecko into the emulator
+            # and then restarting B2G tickles some bug in the emulator/b2g
+            # that intermittently causes B2G to fail to restart.  To work
+            # around this in TBPL runs, we will fail gracefully from this
+            # error so that the mozharness script can try the run again.
+
+            # This string will get caught by mozharness and will cause it
+            # to retry the tests.
+            print "Error installing gecko!"
+
+            # Exit without a normal exception to prevent mozharness from
+            # flagging the error.
+            sys.exit()
+
     def _send_message(self, command, response_key, **kwargs):
         if not self.session and command not in ('newSession', 'getStatus'):
             raise MarionetteException(message="Please start a session")
@@ -398,7 +421,7 @@ class Marionette(object):
 
         return unwrapped
 
-    def execute_js_script(self, script, script_args=None, timeout=True, new_sandbox=True, special_powers=False):
+    def execute_js_script(self, script, script_args=None, async=True, new_sandbox=True, special_powers=False):
         if script_args is None:
             script_args = []
         args = self.wrapArguments(script_args)
@@ -406,7 +429,7 @@ class Marionette(object):
                                       'value',
                                       value=script,
                                       args=args,
-                                      timeout=timeout,
+                                      async=async,
                                       newSandbox=new_sandbox,
                                       specialPowers=special_powers)
         return self.unwrapValue(response)
